#! /usr/bin/perl
use warnings;
use strict;

print << "EOF";

autopp v1.0 - automatic data pre-processing program

Choose one of the following options:

A) create parameter file
B) auto boxing
C) change bcrd
D) interpolation
E) tif to pif
F) remove blemish and linear gradient, and normalization
G) B + F
I) replace a common string
J) change the centers in paramter files
K) change bcrd and parameter files based on pft result
L) compare the difference of orientation, center and correlation coefficients
M) autobox micrographs with emma radius, thin layer value and iteration
N) change the file suffix
O) rebox the boxed images
P) change the center of parameter file based on the map dimension changing
Q) quit
R) convert multiple pif files to mrc files
T) truncate multiple maps into same dimensions
U) Use roughly fitted pdb as guides to refit into EM density
V) normalize the boxed images
W) use pifmapmask to obtain multiple pif map based on multiple pdb files
Y) convert multiple mrc files to pif files
Z) convert multiple pif files to ccp4 files
1) change the file prefix
2) tile the scanned micrograph

EOF

print "Enter your choice: ";
chomp(my $choice = <STDIN>);
$choice =~ tr/a-z/A-Z/;

SWITCH: {
    if ($choice eq 'A') { optionA(); last SWITCH; }
    if ($choice eq 'B') { optionB(); last SWITCH; }
    if ($choice eq 'C') { optionC(); last SWITCH; }
    if ($choice eq 'D') { optionD(); last SWITCH; }
    if ($choice eq 'E') { optionE(); last SWITCH; }
    if ($choice eq 'F') { optionF(); last SWITCH; }
    if ($choice eq 'G') { optionG(); last SWITCH; }
    if ($choice eq 'I') { optionI(); last SWITCH; }
    if ($choice eq 'J') { optionJ(); last SWITCH; }
    if ($choice eq 'K') { optionK(); last SWITCH; }
    if ($choice eq 'L') { optionL(); last SWITCH; }
    if ($choice eq 'M') { optionM(); last SWITCH; }
    if ($choice eq 'N') { optionN(); last SWITCH; }
    if ($choice eq 'O') { optionO(); last SWITCH; }
    if ($choice eq 'P') { optionP(); last SWITCH; }
    if ($choice eq 'Q') { optionQ(); last SWITCH; }
    if ($choice eq 'R') { optionR(); last SWITCH; }
    if ($choice eq 'T') { optionT(); last SWITCH; }
    if ($choice eq 'U') { optionU(); last SWITCH; }
    if ($choice eq 'V') { optionV(); last SWITCH; }
    if ($choice eq 'W') { optionW(); last SWITCH; }
    if ($choice eq 'Y') { optionY(); last SWITCH; }
    if ($choice eq 'Z') { optionZ(); last SWITCH; }
    if ($choice eq '1') { option1(); last SWITCH; }
    if ($choice eq '2') { option2(); last SWITCH; }

    print "Invalid choice $choice - Please try again\n";
    exit;		
}

print "Good bye\n";

# --------------------------------------------------------

sub optionA {

    print "It will read in a text file. ";
    print "Each line of this text file should contain,\n"; 
    print "micrograph number and defocus level";
    print "(tab delimited, for example: 1234 2.543)\n";
    print "Then the file micrograph.dat_000 will ";
    print "be produced in the appointed directory!\n";
    
    print "Enter the text file name:\n";
    chomp(my $txtfile = <stdin>);
    open INPUT, "<$txtfile";
    
    my (@microg, $ampcontrast);
    
    while(my $line = <INPUT>) {
	chomp($line);
	push @microg, $line;			
    }
    
    print "Enter pixel size(angstrom/pixel):\n";
    chomp(my $pixel = <stdin>);
    
    print "Enter 2 for 200Kev or enter 3 for 300Kev microscope:\n";
    chomp(my $scope = <stdin>);
    
    if($scope == 2) {
	$scope = '200000.00';
	$ampcontrast = '0.07';
    } elsif($scope == 3) {
	$scope = '300000.00';
	$ampcontrast = '0.05';
    } else {
	print "Your input is wrong!Exit!\n";
	exit;	
    }
    
    print "Enter the abolute path for the directory";
    print "of your boxed image file? (ends with \/)\n";
    print "for example: /bio/laevo4/xdy/civ/pifimage2/\n";

    chomp(my $path = <stdin>);
    
    print "Enter the suffix of boxed image file:\n";
    print "for example: your image file name is ";
    print "8050-raw-grad.pif, you need to input:\n";
    print "-raw-grad.pif (do not include \*)\n\n";
    chomp(my $suff = <stdin>);
    
    print "Enter the abolute path for the directory";
    print "you want to store prameter data file? (ends with \/)\n";
    print "for example: /bio/laevo4/xdy/civ/data/\n";
    chomp(my $datapath = <stdin>);
    
    for (my $i=0; $i<scalar(@microg); $i++) {
	my ($picnum, $def) = split /\t/,$microg[$i];
	
	my $dat = "${datapath}${picnum}.dat_000";

	open OUTPUT, ">$dat";
	
	my $imgfile = "${path}${picnum}${suff}";
	print OUTPUT "$imgfile\n";
	print OUTPUT "$pixel, 1, $scope, $ampcontrast, ";
	print OUTPUT "$def, $def, 0.000, 2.00\n";
	close OUTPUT;
    }		
}

sub optionB {

    print "This program will read all the scanned micrograph pif\n";
    print "images and the corresponding bcrd file and then write\n";
    print "out the raw boxed images\n";
    print "WARNING: PREVIOUS BOXED IMAGES WILL BE DELETED!\n";
    
    my @bcrdarr = <*.bcrd>;
    
    if(scalar(@bcrdarr) == 0) {
	print "No bcrd files are found! Exit!\n";
	exit;	
    }
    
    print "Enter apodize mode:(gaussian, exponential, linear)\n";
    chomp(my $mode = <STDIN>);
    
    print "Enter box radius:\n";
    chomp(my $boxrad = <STDIN>);
    print "Enter feather radius:\n";
    chomp(my $feather_rad = <STDIN>);
    
    for (my $i=0; $i<scalar(@bcrdarr); $i++) {
	my $pif = $bcrdarr[$i];
	$pif =~  s/bcrd/pif/i;
	
	if (-e $pif) {
	    my $boxfile = $pif;
	    $boxfile =~ s/\.pif/\_box\.pif/i;
	    if(-e $boxfile) {
		unlink($boxfile);
	    }
	    
	    my $robcom = "xbatch robem -pre $pif -BOX $bcrdarr[$i] ";
	    $robcom .= "$mode 0 $boxrad $feather_rad";
	    
	    print "$robcom\n";
	    my $tmp = `$robcom`;		
	    chk_file($boxfile);	
	}	
    }
}

sub optionC {
    print "All bcrd files (ending with bcrd) in this ";
    print "directory will be changed!\n";
    print "New bcrd fiels will be written into .\/tmpbcrd directory!\n";
    print "new X coordinate = old X coordinate * ratio + move\n";
    print "new Y coordinate = old Y coordinate * ratio + move\n";
    
    print "Enter the ratio (can be zero if you just want to translate)!\n";
    chomp(my $ratio = <STDIN>);
    $ratio = sprintf("%.3f", $ratio);
    print "Enter move:\n";
    chomp(my $move = <STDIN>);
    $move = sprintf("%.3f", $move);
    my @filearr = <*.bcrd>;

    if (scalar(@filearr) == 0) {
	print "No bcrd file are found!exit!\n";
	exit;	
    }
    unless(-e "tmpbcrd") {
	mkdir ("tmpbcrd",0700) || die "Cannot make the directory tmpcent!\n";
    }
    
    for (my $i=0; $i<scalar(@filearr); $i++) {
	open IN, "<$filearr[$i]";
	my @tmparr = <IN>;
	close IN;
	my $version = 1.0;
	
	chomp(@tmparr);
	open OUT, ">./tmpbcrd/$filearr[$i]";
	if ($tmparr[0] =~ m/BCRD V2.00/i) {
	    $version = 2.0;
	}
	for (my $j=0; $j<scalar(@tmparr); $j++) {
	    if($version == 1.0 && $j < 3) {
		print OUT "$tmparr[$j]\n";	
	    } elsif ($version == 2.0 && $j < 5) {
		print OUT "$tmparr[$j]\n";
	    } else {
		my ($xcord, $ycord) = split / /,$tmparr[$j];
		$xcord = $xcord * $ratio + $move;
		$xcord = sprintf("%.0f", $xcord);
		
		$ycord = $ycord * $ratio + $move;
		$ycord = sprintf("%.0f", $ycord);
		
		print OUT "$xcord $ycord\n";	
	    }
	}
	close OUT;
    }
}

sub optionD {
    print "All pif files in current directory will be interpolated!\n";
    print "Enter the original pixel value (angstrom)!\n";
    chomp(my $oripixel = <STDIN>);
    print "Enter output pixel value (angstrom):\n";
    chomp(my $outpixel = <STDIN>);
    
    my @filearr = <*.pif>;
    
    if (scalar(@filearr) == 0) {
	print "No pif images file are found!\n";
	exit;	
    }
    
    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $tmp = `xbatch robem -pre $filearr[$i] -interp $oripixel $outpixel`;
	my $file = $filearr[$i];
	$file =~ s/\.pif/\_interp\.pif/i;
	chk_file($file);	
    }
}

sub optionE {
    print "All tif files in current directory will be converted to pif!\n";
    
    my @file = <*.tif>;
    if (scalar(@file) == 0) {
	print "No tif files are found!exit!\n";
	exit;	
    }

    print "Enter your avg factor:1, 2, or 3?";
    chomp(my $binf = <stdin>);
    for(my $i=0; $i<scalar(@file); $i++) {
	my $pifname = $file[$i];
	$pifname =~ s/\.tif/\.pif/i;
	#my $tmp = `echo $file[$i] >tmp.auto_uniconv_$i`;
	my $tmp = `echo $binf >tmp.auto_uniconv_$i`;
	#$tmp = `echo $pifname >>tmp.auto_uniconv_$i`;
	print "Converting $file[$i]-->$pifname...";
	$tmp = `tiff2pif $file[$i] $pifname < tmp.auto_uniconv_$i`;
	print "done\n";
    }

    my $tmp = `rm tmp.auto_uniconv_*`;
    print "\n\nAll the tif files have been converted to pif files, ";
    print "please check!\n";
}

sub optionF {

    print << "EOF";

This option applies blemish removal, linear gradient removal, and
normalization to a set of boxed image files. The list of files to be
modified can be specified using UNIX wildcards and full or relative
path names. For example, all of the following are legal

   /dir1/file1
   ../../dir2/file*
   dir3/file[2-7].*
   /dir4/file1 /dir5/sub1/file2 /dir6/sub2/file*

EOF

    print "Enter list of files to be modified: ";
    my $list = <>;
    my @files = glob("$list");
    if(@files <= 0) {
	print "No files were specified - exiting program\n";
	exit;
    }
    
    print "\n";
    print "Enter 'yes' to specify mean and stddev values to be\n";
    print "used in normalization of data. Otherwise values calculated\n";
    print "from image data. Default behavior (suggested) is 'no': ";
    chomp(my $specify_values = <>);

    my $normval = 0.0;
    my $stdval  = 0.0;

    if(defined $specify_values && $specify_values =~ /^y/i) {
	print "Enter the mean value: ";
	chomp($normval = <>);
	print "Enter the stddev value: ";
	chomp($stdval = <>);
    } else {
	unlink('trashnorm.txt');
	for (my $i=0; $i<scalar(@files); $i++) {
	    my $com = "xbatch robem -pre $files[$i] -DUMPNORM trashnorm.txt";
	    print $com;
	    my $tmp = `$com`;
	}
	
	open IN, "<trashnorm.txt";
	my @lines = <IN>;
	close IN;
	
	for (my $i=0; $i<scalar(@lines); $i++) {
	    if($lines[$i] =~ /mean\[(.*)\]\s+stDev\[(.*)\]/i) {
		my $thisnorm = $1;
		my $thisstd = $2;
		$normval += $thisnorm;
		$stdval += $thisstd;	
	    }	
	}
	
	$normval = $normval/scalar(@lines);
	$stdval = $stdval/scalar(@lines);
    }
    
    print "normalization value = $normval\n";
    print "standard deviation  = $stdval\n";
    print "\n";

    for (my $i=0; $i<scalar(@files); $i++) {
	my $robcom = "xbatch robem -pre $files[$i] -rmblem 0.0";
	print "$robcom\n";
	my $tmp = `$robcom`;
	my $fixfile = $files[$i];
	$fixfile =~ s/\.pif/\_fix\.pif/i;
	chk_file($fixfile);
	$tmp = "xbatch robem -pre $fixfile -lingrad -norm $normval $stdval";
	print "$tmp\n";
	$tmp = `$tmp`;
	
	my $normfile = $fixfile;
	$normfile =~ s/\.pif/\_norm\.pif/i;
	chk_file($normfile);
	
	if(-e $fixfile) {
	    unlink($fixfile);
	}		
    }
}

sub optionG {
    print "This program will read all the scanned microraph ";
    print "pif images and the corresponding bcrd file\n";
    print "write out the raw boxed images and then\n";
    print "remove blemish, remove linear gradient and do the normalization!\n";
    print "WARNING: PREVIOUS BOXED IMAGES WILL BE DELETED!\n";
    my @bcrdarr = <*.bcrd>;
    if(scalar(@bcrdarr) == 0) {
	print "No bcrd files are found! Exit!\n";
	exit;	
    }

    print "Enter apodize mode:(gaussian, exponential, linear)\n";
    chomp(my $mode = <STDIN>);

    print "Enter box radius:\n";
    chomp(my $boxrad = <STDIN>);

    print "Enter feather radius:\n";
    chomp(my $feather_rad = <STDIN>);

    print "Enter normalization value (input 0.0 to let the ";
    print "program to calculate the normalization value):\n";

    chomp(my $normval = <STDIN>);

    print "Enter Standard deviation value(input 0.0 to let the ";
    print "program to calculate the value):\n";
    chomp(my $stdval = <STDIN>);
    
    my @boxfiles;
    
    for (my $i=0; $i<scalar(@bcrdarr); $i++) {
	my $bcrd = $bcrdarr[$i];
	my $pif = $bcrd;
	$pif =~  s/\.bcrd/\.pif/i;
	
	my $boxfile = $pif;
	$boxfile =~ s/\.pif/\_box\.pif/i;
	if(-e $boxfile) {
	    unlink($boxfile);
	}
	my $robcom = "xbatch robem -pre $pif -BOX $bcrd $mode 0 $boxrad ";
	$robcom .= "$feather_rad";
	print "$robcom\n";
	my $tmp = `$robcom`;
	chk_file($boxfile);
	push (@boxfiles, $boxfile);
    }
    
    if($normval == 0.0 || $stdval == 0.0) {	
	# program will calculate the average value    
	unlink('trashnorm.txt');
	my $i = 0;
	while($i < scalar(@boxfiles)) {
	    my $com = "xbatch robem -pre $boxfiles[$i] -DUMPNORM ";
	    $com .= "trashnorm.txt";
	    my $tmp = `$com`;
	}
	open IN, "<trashnorm.txt";
	my @lines = <IN>;
	close IN;
	
	for (my $i=0; $i < scalar(@lines); $i++) {
	    if($lines[$i] =~ /mean\[(.*)\]\s+stDev\[(.*)\]/i) {
		my $thisnorm = $1;
		my $thisstd = $2;
		$normval += $thisnorm;
		$stdval += $thisstd;	
	    }	
	}
	
	$normval = $normval/scalar(@lines);
	$stdval = $stdval/scalar(@lines);
    }
    
    print "normalization value is $normval!\n";
    print "standard deviation is $stdval!\n";	
    for (my $i=0; $i<scalar(@boxfiles); $i++) {
	my $fixfile = $boxfiles[$i];
	$fixfile =~ s/\.pif/\_fix\.pif/i;
	if(-e $fixfile) {
	    unlink($fixfile);
	}		
	my $robcom = "xbatch robem -pre $boxfiles[$i] -rmblem 0.0";
	print "$robcom\n";
	my $tmp = `$robcom`;
	my $normfile = $fixfile;
	$normfile =~ s/.pif/\_norm\.pif/i;
	if(-e $normfile) {
	    unlink($normfile);
	}
	$tmp = `xbatch robem -pre $fixfile -lingrad -norm $normval $stdval`;
	unlink($boxfiles[$i]);
	unlink($fixfile);
	chk_file($normfile);
    }
}

sub optionI {

    print << "EOF";
    
This option does a global replace of an arbitrary string in a set of
text files. The list of files to be modified can be specified using
UNIX wildcards and full or relative path names. For example, all of
the following are legal

   /dir1/file1
   ../../dir2/file*
   dir3/file[2-7].*
   /dir4/file1 /dir5/sub1/file2 /dir6/sub2/file*

Original files will be overwritten - if in doubt create a backup. You
must have both read and write permissions for the files that you want
to modify. All files that are not ordinary text files will be ignored.

EOF
    
    print "Enter list of files to be modified: ";
    my $list = <>;
    
    print "Enter string to be replaced: ";
    chomp(my $oldstr = <>);
    
    print "Enter replacement string: ";
    chomp(my $newstr = <>);
    
# Expand $list into array of files
    my @files = glob("$list");
    print "\n";
    
# Test to make sure that files were found
    if(@files <= 0) {
	print "No files were specified - exiting program\n";
	exit;
    }
    
    foreach my $file (@files) {
	if(! -r $file) {
	    print "$file is not readable by user or group\n";
	    next;
	} elsif (! -w $file) {
	    print "$file is not writeable by user or group\n";
	    next;
	} elsif (! -T $file) {
	    print "$file is not a plain text file\n";
	    next;
	} else {
	    print "Modifying $file\n";
	}
	
	# Read all records in file into array @lines
	open(FH, "+<$file");
	my @lines = <FH>;
	
	# Truncate file
	seek(FH, 0, 0);
	truncate(FH, 0);
	
	# Write modified records to file
	foreach my $line (@lines) {
	    $line =~ s/$oldstr/$newstr/;
	    print FH $line;
	}
	close(FH);
    }
}

sub optionJ {
    print "This program will change the center x, y value ";
    print "of the parameter file,\n"; 
    print "and write the output file in \.\/tmpcent/\n";

    unless (-e "tmpcent") {
	mkdir ("tmpcent",0700) ||die "Cannot make the directory tmpcent!\n";
    }
    
    print "Enter the suffix of the parameter file, ";
    print "for example .dat_010 (do not include *):\n";
    chomp(my $suffix = <STDIN>);
    print "Enter diffx (new x = old x + diffx):\n";
    chomp(my $diffx = <STDIN>);
    print "Enter diffy (new y = old y + diffy):\n";
    chomp(my $diffy = <STDIN>);
    
    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No parameter files are found!\n";
	exit;	
    } 
     
    for (my $i=0; $i<scalar(@filearr); $i++) {
	
	open IN, "<$filearr[$i]" || die "cannot open the file $filearr[$i]\n";
	my @lines = <IN>;
	close IN;
	my $outfile = ".\/tmpcent\/$filearr[$i]";
	open OUT, ">$outfile" || 
	    die "cannot open the file $outfile for writing!\n";

        for (my $j=0; $j<scalar(@lines); $j++) {
	    if($j < 2) {
		print OUT $lines[$j];
	    } else {
		my $tmpline = $lines[$j];
		$tmpline =~ /(\d+\.\d+)\,\s+(\d+\.\d+)\, 1.000\,/;
		my $newx = $1 + $diffx;
		$newx = sprintf("%.3f", $newx);
		my $newy = $2 + $diffy;
		$newy = sprintf("%.3f", $newy);
		my $newline;
		
		if($newx < 100.0 && $newx >= 10.0) {
		    $newx = ' ' . $newx;
		}
		if($newy < 100.0 && $newx >= 10.0) {
		    $newy = ' ' . $newy;
		}

		$newline = $` . $newx . ',' . $newy . ', 1.000,' . $';
		
		print OUT $newline;
	    }	    
	}	
	close OUT;	
    }
}

sub optionK {
    my ($datpath, $suffix, $scandir, $tmpdir, $outerboxrad);
    
    print "This program will read the parameter files produced by pft, \n";
    print "change the bcrd files and parameter files so that the ";
    print "center of the boxed particle is close to \n";
    print "geometric centers!\n";
    
    print "Enter the outer box radius, ex, if image is 511^2, ";
    print "the radius should be 255!\n";
    chomp($outerboxrad = <STDIN>);
    
    print "Enter the path of dat files!\n";
    chomp($datpath = <STDIN>);
    
    print "Enter the suffix of the dat files:\n";
    chomp($suffix = <STDIN>);
    
    $tmpdir = "${datpath}originaldat/";
    unless(-e $tmpdir) {
	create_dir($tmpdir);
    }	
    
    print "Enter the full path of scanned image (bcrd files ";
    print "should exist in the same directory)ending with \/!\n";
    chomp($scandir = <STDIN>);
    chk_dir($scandir);
    
    my $pftboxdir = "${scandir}changedbcrd/";
    
    unless(-e "$pftboxdir") {
	mkdir ("$pftboxdir",0755) ||die "Cannot make the directory pftboxd!\n";
    }
    
    my @filearr = <$datpath*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No dat file are found!\n";
	exit;	
    }
    
    for (my $i=0; $i < scalar(@filearr); $i++) {
	open IN, "<$filearr[$i]"; #open this datfile
	my @indat_arr = <IN>;
	close IN;
	
	my $bcrdin = get_bcrd($filearr[$i], $scandir);	#get bcrd file 
	if(check_file($bcrdin) == 0) {
	    next;	
	}
	print $bcrdin;
	
	my $scanimg = $bcrdin;			#get scanned image file
	$scanimg =~ s/\.bcrd/\.pif/i;
	if(check_file($scanimg) == 0) {
	    next;	
	}
	
	open INBCRD, "<$bcrdin";		#open original bcrd file
	my @inbcrd_arr = <INBCRD>;
	chomp(@inbcrd_arr);
	close INBCRD;
	
	my $bcrdout = $bcrdin;
	print "$scandir\n";
	print "$pftboxdir\n";
	
	$bcrdout =~ s/$scandir/$pftboxdir/i;
	print $bcrdout;

	#my $boxflnImg =  $bcrdout;
	#the final image file name
	#$boxflnImg =~ s/\.bcrd/\_box\_fix\_norm\.pif/i;	
	#print $boxflnImg;
	
	# move the original dat file to tmp dir
	my $mov = "mv $filearr[$i] $tmpdir\.";	
	print "\n$mov\n";
	my $tmp = `$mov`;
	
	# write out the new bcrd file
	open OUTBCRD, ">$bcrdout" ||die "cannot write out new bcrd!Exit!\n";
	# new dat file, same name as the old one
	open OUTDAT, ">$filearr[$i]" ||die "cannot write out new dat file!Exit!\n";
	
	my $origin = $outerboxrad; #what is the geometry center
	
	# copy variables over, just to keep their names, it is stupid!
	my $boxradius = $origin;  
	
	
	my $bcrdindex = 1;
	for (my $j=0; $j < scalar(@indat_arr); $j++) {
	    if($j == 0) {
		print OUTDAT "$indat_arr[$j]";
		print OUTBCRD "$inbcrd_arr[$j]\n";	
		
	    } elsif($j == 1) {
		print OUTDAT "$indat_arr[$j]";
		print OUTBCRD "$inbcrd_arr[$j]\n";	
		print OUTBCRD "$inbcrd_arr[$j + 1]\n";	 
		if($inbcrd_arr[0] =~ m/BCRD/i) {
		    print OUTBCRD "$inbcrd_arr[$j + 2]\n";
		    print OUTBCRD "$inbcrd_arr[$j + 3]\n";
		    $bcrdindex = 3;
		}
	    } else {
		# original box cord
		my ($xcord, $ycord) = split / /,$inbcrd_arr[$j + $bcrdindex];
		my $tmpline = $indat_arr[$j];
		$tmpline =~ /(\d+)\.(\d+)\,\s+(\d+)\.(\d+)\, 1.000\,/;
		
		my $xcent_int = $1;
		my $ycent_int = $3;
		
		$xcord = $xcord + ($xcent_int - $origin);
		$ycord = $ycord - ($ycent_int - $origin);
		print OUTBCRD "$xcord $ycord\n";
		
		my $newx = $origin . '.' . $2;
		my $newy = $origin . '.' . $4;
		my $newline;
		if($newy >= 100.0) {
		    $newline = $` . $newx . ',' . $newy . ', 1.000,' . $';
		} elsif ($newy >= 10.0) {
		    $newline = $` . $newx . ',' . ' ' . $newy . ', 1.000,' . $';
		} else {
		    $newline = $` . $newx . ',' . '  ' . $newy . ',1.000,' . $';
		}
		print OUTDAT $newline;
	    }
	}
    }
}

sub optionL {
    print "This program will read in a two sets of parameter files of pft\n"; 
    print "and compare the difference of orientation, center and ";
    print "correlation coefficients!\n";
    
    print "Enter 3 digits of dat suffix for first batch of ";
    print "parameter file, for example: 001?     ";

    my $f1 = <stdin>;
    chomp($f1);
    my @prevfile = <*$f1>;

    if(scalar(@prevfile) == 0) {
	print "No files are found! Exit!\n";
	exit;	
    }
    
    print "Enter 3 digits of dat suffix for second batch of parameter ";
    print "file, for example: 002?     ";
    my $f2 = <stdin>;
    chomp($f2);
    my @currfile = <*$f2>;
    if(scalar(@currfile) == 0) {
	print "No files are found! Exit!\n";
	exit;	
    }
    
    print "Enter your output file name:\n";
    my $result = <stdin>;
    chomp($result);
    
    my ($diftheta, $difphi, $difomega, $difx, $dify, $pindex);
    format CMPRES =
 	@####,@####.###,@####.###,@####.###,@####.###,@####.###
	$pindex,$diftheta,$difphi,$difomega,$difx,$dify
.
    open CMPRES, ">$result" || die "cannot write out the file $result!\n";

    for (my $i=0; $i<scalar(@prevfile); $i++) {
	my $file1 = $prevfile[$i];
	chomp($file1);
	my $file2 =$file1;
	$file2 =~ s/$f1/$f2/;
	
	open IN, "<$file1";
	my @lines1 = <IN>;
	if(scalar(@lines1 <= 2)) {
	    next;
	}
	close IN;
    
	open IN, "<$file2";
	my @lines2 = <IN>;
	if(scalar(@lines2 <= 2)) {
	    next;
	}
	close IN;

	shift(@lines1);
	shift(@lines1);
	shift(@lines2);
	shift(@lines2);
	
	print CMPRES "*********$file1**********$file2**************\n";
	my @sortlines1 = sort by_index @lines1;
	my @sortlines2 = sort by_index @lines2;
	my ($avgtheta, $avgphi, $avgomega, $avgx, $avgy);

	my $count = scalar(@lines1) > scalar(@lines2) ? 
	    scalar(@lines1): scalar(@lines2);

	for (my $j=0; $j<$count; $j++) {
	    
	    my @arr1 = split /,/, $sortlines1[$j];
	    my @arr2 = split /,/, $sortlines2[$j];
	    
	    $diftheta = $arr2[1] - $arr1[1];
	    $diftheta = sprintf("%  3.3f", $diftheta);
	    $avgtheta += abs($diftheta);
	    
	    $difphi = $arr2[2] - $arr1[2];
	    $difphi = sprintf("%  3.3f", $difphi);
	    $avgphi += abs($difphi);
	    
	    $difomega = $arr2[3] - $arr1[3];
	    $difomega = sprintf("%  4.3f", $difomega);
	    $avgomega += abs($difomega);
	    
	    $difx = $arr2[4] - $arr1[4];
	    $difx = sprintf("%  5.3f", $difx);
	    $avgx += abs($difx); 
	    
	    $dify = $arr2[5] - $arr1[5];
	    $dify = sprintf("%  4.3f", $dify);
	    $avgy += abs($dify);
	    
	    $pindex = $arr1[0];
	    
	    write CMPRES;
	}
	
	$avgtheta = $avgtheta/$count;
	$avgtheta = sprintf("%  4.3f", $avgtheta);
	$avgphi = $avgphi/$count;
	$avgphi = sprintf("%  4.3f", $avgphi);
	$avgomega = $avgomega/$count;
	$avgomega = sprintf("%  4.3f", $avgomega);
	$avgx = $avgx/$count;
	$avgx = sprintf("%  4.3f", $avgx);
	$avgy = $avgy/$count;
	$avgy = sprintf("%  4.3f", $avgy);
	
	$pindex = 0;
	$diftheta = $avgtheta;
	$difphi = $avgphi;
	$difomega = $avgomega;
	$difx = $avgx;
	$dify = $avgy;
	
	print ".";
	print CMPRES "	   #########The averaged absolute difference########\n";
	write CMPRES;
	print CMPRES "	   #################################################\n\n\n";
	}
   print "\n";
   print "Success, please see $result \n";
}

sub optionM {
    print "This program will autobox all the scanned micrographs ";
    print "(suffix is pif) in current directory!\n"; 
    
    print "Enter emma radius: ";
    my $emma = <stdin>;
    chomp($emma);
    print "Enter box radius: ";
    my $boxrad = <stdin>;
    chomp($boxrad);
    print "Enter feather radius: ";
    my $feather = <stdin>;
    chomp($feather);
    print "Enter thin layer value: ";
    my $thin = <stdin>;
    chomp($thin);
    print "Enter box refine interation: ";
    my $iter = <stdin>;
    chomp($iter);
    
    my @scanned = <*pif>;
    if(scalar(@scanned) == 0) {
	print "No pif files are found! Exit!\n";
	exit;	
    }

    for (my $i=0; $i<scalar(@scanned); $i++) {
	my $robcom = "xbatch robem -pre $scanned[$i] -AUTOBOX gaussian ";
	$robcom .= "$emma $boxrad $feather $thin $iter";
	print "$robcom\n";
	my $tmp = `$robcom`;		
    } 
}

sub optionN {
    print "This program will change the file's suffix in this directory, \n";
    print "use it with CAUTION!BACKUP your old files first! \n"; 
    
    print "Enter the suffix of the files you want to change: ";
    chomp(my $oldsuff = <stdin>);
    print "Enter new suffix you want these files to be: ";
    chomp(my $newsuff = <stdin>);
    
    my @files = <*$oldsuff>;
    if(scalar(@files) == 0) {
	print "No files with suffix $oldsuff are found! Exit!\n";
	exit;	
    }

    for (my $i=0; $i<scalar(@files); $i++) {
	my $oldfile = $files[$i];
	my $newfile = $oldfile;
	$newfile =~ s/$oldsuff/$newsuff/;
	
	my $tmp = `mv $oldfile $newfile`;
	print "$oldfile ==> $newfile   \n";		
    } 
}

sub optionO {
    print "All the boxed image files in currrent directory will be reboxed:\n";
    print "please enter the suffix of your boxed image file, for example: \n";
    print "-raw.pif (do not include *)\n";

    chomp(my $suffix = <STDIN>);
    print "Enter the new inner box radius:\n";
    chomp(my $boxrad = <STDIN>);
    print "Enter the feather (outer) radius (should be bigger than ";
    print "inner box radius:\n";
    chomp(my $featherrad = <STDIN>);

    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No boxed images file are found!\n";
	exit;	
    } 

    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $outfile = $filearr[$i];
	$outfile =~ s/\.pif/\_rebox\.pif/;
	my $robcom = "xbatch robem -pre $filearr[$i] -REBOX gaussian ";
	$robcom .= "$boxrad $featherrad -OUTFILE $outfile";
	print "$robcom\n";
	my $tmp = `$robcom`;		
    } 
}

sub optionP {
    print "This program will change the center x, y value of the parameter file,\n"; 
    print "and write the output file in \.\/tmpcent/\n";
    unless (-e "tmpcent") {
	mkdir ("tmpcent",0700) ||die "Cannot make the directory tmpcent!\n";
    }
    print "Enter the suffix of the parameter file, for example ";
    print ".dat_010 (do not include *):\n";
    chomp(my $suffix = <STDIN>);
    print "Enter the old theoretical center value:\n";
    chomp(my $old = <STDIN>);
    print "Enter the new theoretical center value:\n";
    chomp(my $new = <STDIN>);
    
    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No parameter files are found!\n";
	exit;	
    }
    
    for (my $i=0; $i < scalar(@filearr); $i++) {
	open IN, "<$filearr[$i]" || die "cannot open the file $filearr[$i]\n";
	my @lines = <IN>;
	close IN;
	my $outfile = ".\/tmpcent\/$filearr[$i]";
	open OUT, ">$outfile" || die "cannot open the file $outfile for writing!\n";

	for (my $j=0; $j < scalar(@lines); $j++) {
	    if($j < 2) {
		print OUT $lines[$j];
	    } else {
		my $tmpline = $lines[$j];
		$tmpline =~ /(\d+\.\d+)\,(\d+\.\d+)\,/;
		
		my $difx = $1 - $old;
		my $dify = $2 - $old;
		my $ratio = ($new+1)/($old+1);
		my $newx = $difx * $ratio + $new;
		my $newy = $dify * $ratio + $new;
		
		$newx = sprintf("%.3f", $newx);
		$newy = sprintf("%.3f", $newy);
		my $newline = $` . $newx . ',' . $newy . ',' . $';
		print OUT $newline;
	    }
	}
	close OUT;	
    }
}

sub optionQ {
    print "Come back next time!\n";
}

sub optionR {
    print "This program will convert multiple pif files to mrc files!\n"; 
    print "Enter the suffix of the map file, for example .pif ";
    print "(do not include *):\n";
    chomp(my $suffix = <STDIN>);
    
    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No pif files are found!\n";
	exit;	
    }

    my $trash = 'trash.txt';
    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $mapfile = $filearr[$i];
	my $outfile = $mapfile;
	$outfile =~ s/\.pif/\.mrc/i;
	open IN, ">$trash";
	print IN "$mapfile\n";
	print IN "$outfile\n";
	close IN;
	my $com = 'pif2mrc < trash.txt';
	print "$com\n";
	my $tmp = `$com`;		
	check_file($outfile);	
	print "******** $outfile is created!\n";
    }
    unlink ($trash);
}

sub optionT {
    print "This program will truncate multiple maps.\n"; 
    
    print "Enter the suffix of the map file, for example ";
    print ".pif (do not include *):\n";
    chomp(my $suffix = <STDIN>);
    
    print "Enter the beginx:\n";
    chomp(my $beginx = <STDIN>);
    
    print "Enter the endx:\n";
    chomp(my $endx = <STDIN>);
    
    print "Enter the beginy:\n";
    chomp(my $beginy = <STDIN>);
    
    print "Enter the endy:\n";
    chomp(my $endy = <STDIN>);
    
    print "Enter the beginz:\n";
    chomp(my $beginz = <STDIN>);
    
    print "Enter the endz:\n";
    chomp(my $endz = <STDIN>);
    
    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No map files are found!\n";
	exit;	
    }
    
    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $mapfile = $filearr[$i];
	my $outfile = $mapfile;
	$outfile =~ s/\.pif/\-cut\.pif/i;

	my $robcom = "xbatch robem -pre $mapfile -TRUNC $beginx $endx ";
	$robcom .= "$beginy $endy $beginz $endz -OUTFILE $outfile";

	print "$robcom\n";
	my $tmp = `$robcom`;		
	check_file($outfile);	
	print "******** $outfile is created!\n";
    }
}

sub optionU {
    print "This program will use all the pdbs in this directory to\n";
    print "cut out a cube of pif density for each pdb file!\n"; 
    print "Then, each small pif will be converted to ccp4 and situs,\n";
    print "and then do the fitting.\n";
    print "each fitting will be stored in its own sub directory!\n";
    
    print "Enter the map file name:\n";
    chomp(my $bigmap = <STDIN>);
    
    print "Enter the dimension of the map, EX: 511:\n";
    chomp(my $mapdim = <STDIN>);
    
    print "Enter the pixel size:\n";
    chomp(my $pixelsize = <STDIN>);
    
    print "Enter the buffer radius in pixels to the center of the pdb molecule:\n";
    chomp(my $buffrad = <STDIN>);
    
    print "enter 3 atoms to determine the center of the pdb!\n";
    print "for example: A 123 B 123 C 123\n";
    chomp (my $atom3 = <STDIN>);
    
    print "Enter the full dir and name of the pdb file you ";
    print "want to use to do the fitting:\n";
    chomp(my $fitpdb = <STDIN>);
    check_file($fitpdb);
    
    print "Enter the highest resoltuion you want to do the fitting (in A):\n";
    chomp(my $res = <STDIN>);
    
    print "ENter how many explorers you want for one fitting:\n";
    chomp(my $explor = <STDIN>);
    
    
    my @atoms = split / /, $atom3;
    print "$atoms[0]\n";
    print "$atoms[1]\n";
    print "$atoms[2]\n";
    print "$atoms[3]\n";
    print "$atoms[4]\n";
    print "$atoms[5]\n";
    
    my @filearr = <*pdb>;
    if (scalar(@filearr) == 0) {
	print "No pdb files are found!\n";
	exit;	
    }
    
    for (my $i=0; $i < scalar(@filearr); $i++) {
	open PDB, "<$filearr[$i]";
	my @pdb = <PDB>;
	close PDB;
	my $j = 0;
	my $count = 0;
	my ($x1, $x2, $x3, $y1, $y2, $y3, $z1, $z2, $z3);

	while($j < scalar(@pdb) && $count < 3) {
	    
	    if($pdb[$j] !~ m/^ATOM/i) {
		$j++;
		next;
	    }

	    my $chain = substr($pdb[$j],21,1);
	    my $resseqnum = substr($pdb[$j], 22, 4);
	    my $alpha = substr($pdb[$j], 12, 4);
	    
	    #$alpha =~ s/\s+//;
	    $resseqnum =~ s/\s+//;
	    
	    #print "$alpha\n";
	    #print "$pdb[$j]\n";
	    #print "$chain, $resseqnum, $alpha\n";
	    #print "$resseqnum eq $atoms[1] $alpha eq CA $chain eq $atoms[0]\n";
	    
	    if(($resseqnum == $atoms[1]) && ($alpha eq ' CA ') && 
	       ($chain eq $atoms[0]) ) {

		$x1 = substr($pdb[$j],30,8);
		$y1 = substr($pdb[$j],38,8);
		$z1 = substr($pdb[$j],46,8);
		
		#print "$pdb[$j]\n";
		$count++;
		#print "$x1, $y1, $z1\n";
		#print "got 1\n";
	    } 

	    if(($resseqnum == $atoms[3]) && ($alpha eq ' CA ') && 
	       ($chain eq $atoms[2])) {

		$x2 = substr($pdb[$j],30,8);
		$y2 = substr($pdb[$j],38,8);
		$z2 = substr($pdb[$j],46,8);
		
		#print "$pdb[$j]\n";
		$count++;
		#print "$x2, $y2, $z2\n";
		#print "got 2\n";
	    } 

	    if(($resseqnum == $atoms[5]) && ($alpha eq ' CA ') && 
	       ($chain eq $atoms[4])) {

		$x3 = substr($pdb[$j],30,8);
		$y3 = substr($pdb[$j],38,8);
		$z3 = substr($pdb[$j],46,8);
		
		#print "$pdb[$j]\n";
		$count++;
		#print "$x3, $y3, $z3\n";
		#print "got 3\n";
	    } 
	    
	    $j++;    
	}

	if($count < 3) {
	    print "only $count atoms are found in pdb, ";
	    print "not enough to determine the center,exit!\n";
	    exit;
	}
	
	my $centerx = ($x1+$x2+$x3)/3;
	my $centery = ($y1+$y2+$y3)/3; 
	my $centerz = ($z1+$z2+$z3)/3;
	print "center: $centerx, $centery, $centerz\n";
	
	my $beginx = $centerx/$pixelsize-$buffrad + ($mapdim+1)/2;
	my $endx = $centerx/$pixelsize+$buffrad + ($mapdim+1)/2;
	my $beginy = $centery/$pixelsize-$buffrad + ($mapdim+1)/2;
	my $endy = $centery/$pixelsize+$buffrad + ($mapdim+1)/2;
	my $beginz = $centerz/$pixelsize-$buffrad + ($mapdim+1)/2;
	my $endz = $centerz/$pixelsize+$buffrad + ($mapdim+1)/2;
	
	$beginx = sprintf("%.0f", $beginx);
	$endx = sprintf("%.0f", $endx);
	$beginy = sprintf("%.0f", $beginy);
	$endy = sprintf("%.0f", $endy);
	$beginz = sprintf("%.0f", $beginz);
	$endz = sprintf("%.0f", $endz);
	
	#print "$beginx $endx $beginy $endy $beginz $endz\n";
	
	my $dirname = $filearr[$i];
	$dirname =~ s/.pdb//;
	
	unless (-e "$dirname") {
	    mkdir ("$dirname",0700) ||die "Cannot make the sub directory!\n";
	}
	my $outfile = $dirname . '/'. $bigmap;
	$outfile =~ s/\.pif/\-cut\.pif/i;

	my $robcom = "xbatch robem -pre $bigmap -TRUNC $beginx $endx ";
	$robcom .= "$beginy $endy $beginz $endz -OUTFILE $outfile";

	print "$robcom\n";
	my $tmp = `$robcom`;		
	check_file($outfile);	
	print "******** $outfile is created!\n";
	
	my $ccp4file = $outfile;
	$ccp4file =~ s/\.pif/\.ccp4/;
	
	open CCPIN, ">trash.txt";
	print CCPIN "$outfile\n";
	print CCPIN "$ccp4file\n";
	print CCPIN "$pixelsize\n\n";
	close CCPIN;

	my $pif2ccp4com = 'pif2ccp4 < trash.txt';
	$pif2ccp4com = `$pif2ccp4com`;
	check_file($ccp4file);
	print "******** $ccp4file is created!\n";

	open SITUS, ">trash.txt";
	print SITUS "4\n";
	print SITUS "$pixelsize\n";
	close SITUS;

	my $situsfile = $ccp4file;
	$situsfile =~ s/\.ccp4/\.situs/;
	
	my $situscom = "conformat $ccp4file $situsfile < trash.txt";
	$situscom = `$situscom`;
	check_file($situsfile);
	print "******** $situsfile is created!\n";
	$situsfile =~ s/$dirname\///;
	
	chdir($dirname);
	
	my $fitcom = "colores  $situsfile $fitpdb \-explor $explor ";
	$fitcom .= "\-res $res \> situs.log";
	print "$fitcom\n";
	
	$fitcom = `$fitcom`;	
	chdir("\.\.");	
    }
}

sub optionV {
    print "All the boxed image files in currrent directory ";
    print "will be normalized:\n";
    print "please enter the suffix of your boxed image file, for example: ";
    print "-raw.pif (do not include *)\n";
    chomp(my $suffix = <STDIN>);
    print "Enter normalization value:\n";
    chomp(my $normvalue = <STDIN>);
    print "Enter Standard deviation value:\n";
    chomp(my $stdvalue = <STDIN>);
    my @filearr = <*$suffix>;

    if (scalar(@filearr) == 0) {
	print "No boxed images file are found!\n";
	exit;	
    }
    
    print "normalization value is $normvalue!\n";
    print "standard deviation is $stdvalue!\n";
    
    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $tmp = "xbatch robem -pre $filearr[$i] -norm $normvalue $stdvalue";
	print "$tmp\n";
	$tmp = `$tmp`;
	my $normfile = $filearr[$i];
	$normfile =~ s/\.pif/\_norm\.pif/i;
	chk_file($normfile);
    }
}

sub optionW {
    print "All the pdb files in this directory will be ";
    print "considered to mask out a pif map:\n";
    
    my @filearr = <*pdb>;
    if (scalar(@filearr) == 0) {
	print "No pdb files are found!\n";
	exit;	
    }

    print "please enter the pif map file\n";
    chomp(my $pifmap = <STDIN>);
    chk_file($pifmap);
    print "please enter the pixel size\n";
    chomp(my $pixsize = <STDIN>);
    print "please enter the border in pixels:\n";
    chomp(my $border = <STDIN>);
    
    my $trash = 'trash.txt';

    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $pdbfile = $filearr[$i];
	my $outfile = $pdbfile;
	$outfile =~ s/\.pdb/\.pif/i;
	open IN, ">$trash";
	print IN "$border\n";
	print IN "$pifmap\n";
	print IN "$outfile\n";
	print IN "$pixsize\n";
	print IN "$pdbfile\n";
	close IN;
	my $com = 'pifmapmask < trash.txt';
	print "$com\n";
	my $tmp = `$com`;		
	check_file($outfile);	
	print "******** $outfile is created!\n";
    }
    unlink ($trash);
}

sub optionY {
    print "This program will convert multiple mrc files to pif files!\n"; 
    print "Enter the suffix of the map file, for example ";
    print ".mrc (do not include *):\n";
    chomp(my $suffix = <STDIN>);
    
    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No mrc files are found!\n";
	exit;	
    }

    my $trash = 'trash.txt';
    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $mapfile = $filearr[$i];
	my $outfile = $mapfile;
	$outfile =~ s/\.mrc/\.pif/i;
	open IN, ">$trash";
	print IN "$mapfile\n";
	print IN "$outfile\n";
	close IN;
	my $com = 'mrc2pif < trash.txt';
	print "$com\n";
	my $tmp = `$com`;		
	check_file($outfile);	
	print "******** $outfile is created!\n";
    }
    unlink ($trash);
}

sub optionZ {
    print "This program will convert multiple pif files to ccp4 files!\n"; 
    
    print "Enter the suffix of the map file, for example .pif";
    print "(do not include *):\n";
    chomp(my $suffix = <STDIN>);
    
    print "Enter the pixel size\n";
    chomp(my $pixel = <STDIN>);
    
    my @filearr = <*$suffix>;
    if (scalar(@filearr) == 0) {
	print "No pif files are found!\n";
	exit;	
    }

    my $trash = 'trash.txt';
    for (my $i=0; $i<scalar(@filearr); $i++) {
	my $mapfile = $filearr[$i];
	my $outfile = $mapfile;
	$outfile =~ s/\.pif/\.ccp4/i;
	open IN, ">$trash";
	print IN "$mapfile\n";
	print IN "$outfile\n";
	print IN "$pixel\n\n";
	close IN;
	my $com = 'pif2ccp4 < trash.txt';
	print "$com\n";
	my $tmp = `$com`;		
	check_file($outfile);	
	print "******** $outfile is created!\n";
    }
    unlink ($trash);
}

sub option1 {
    
    print "This option changes the prefix for files in this directory\n";
    print "Use with CAUTION and backup your old files first\n";
    
    print "Enter the prefix of the files you want to change: ";
    chomp(my $oldsuff = <stdin>);
    print "Enter new prefix you want these files to be: ";
    chomp(my $newsuff = <stdin>);
    
    my @files = <$oldsuff*>;
    if(scalar(@files) == 0) {
	print "No files with prefix $oldsuff are found! Exit!\n";
	exit;	
    }

    for(my $i=0; $i<scalar(@files); $i++) {
	my $oldfile = $files[$i];
	my $newfile = $oldfile;
	$newfile =~ s/$oldsuff/$newsuff/;
	
	my $tmp = `mv $oldfile $newfile`;
	print "$oldfile ==> $newfile   \n";		
    }     
}

sub option2 {

    print "This option will create a BCRD file of a scanned micrograph\n";
    print "BCRD file can be used to tilt the whole micrograph\n";
    print "WARNING: please back up your previous BCRD file!\n";

    my @pifarr = <*.pif>;
    if(scalar(@pifarr) == 0) {
	print "No pif files are found! Exit!\n";
	exit;	
    }
    
    print "Enter the box dimension (diameter in pixels) default: 201:\n";
    chomp(my $boxdim = <STDIN>);
    if (!$boxdim) {
	$boxdim = 201;
    }
    if($boxdim > 511) {
	$boxdim = 511;
    }
    
    my $tmp = $boxdim % 2;
    my $boxrad;
    if ($tmp == 0) {
	$boxrad = $boxdim / 2 ;
    } else {
	$boxrad = ($boxdim -1) / 2;
	
    }
    
    for (my $i=0; $i<scalar(@pifarr); $i++) {
	my $bcrd = $pifarr[$i];
	$bcrd =~  s/pif/bcrd/i;
	if(-e $bcrd) {
	    print "Existing BCRD files found: $bcrd\n";
	    print "please back up and remove your existing BCRD files!\n";
	    exit;
	} else {
	    my $com = 'pifInfo ' . "$pifarr[$i]";
	    my $output = `$com`;
	    
	    my @info = split /\]\n/, $output;
	    my ($nimg, $numCols, $numRows);
	    
	    for (my $j=0; $j<scalar(@info); $j++) {
		if($info[$j] =~ m/^(\w+)\s+\[(\d*)/) {
		    if($1 eq 'nimg' ) {
			$nimg = $2;
		    }
		    elsif($1 eq 'numCols' ) {
			$numCols = $2;
		    }
		    elsif($1 eq 'numRows' ) {
			$numRows = $2;
		    }
		}
	    }
	    
	    if($nimg == 1) {
		my @boxarr;
		push @boxarr, 'BCRD V2.00';
		push @boxarr, '0';
		push @boxarr, '10000';
		push @boxarr, "$boxrad";
		push @boxarr, "$boxrad";
		
		my $m = 1;
		my $n = 1;
		my $xlen = $boxdim + 30;
		my $ylen = $boxdim + 30;
		my $xytemp = $ylen;
		
		my $xborder = $numCols - $boxrad - 10;
		my $yborder = $numRows - $boxrad - 10;
		my $boxnum = 0;
		
		while ($xlen < $xborder) {
		    $ylen = $xytemp;
		    while ($ylen < $yborder) {
			my $xrad = $xlen - $boxrad;
			my $yrad = $ylen - $boxrad; 
			push @boxarr, "$xrad $yrad";
			$boxnum++;
			$ylen = $ylen + $boxdim;
		    }
		    $xlen = $xlen + $boxdim;
		}
		
		$boxarr[2] = $boxnum;
		
		open BCRD, ">$bcrd" || die "cannot open $bcrd for write!\n";
		
		foreach (@boxarr) {
		    print BCRD "$_\n";	
		}
		
		close BCRD;
	    }
	}	
    }
}
# --------------------------------------------------------

sub chk_file {
    my ($filename) = @_;
    unless(-e $filename) {
	print "Error: $filename is not produced! Please check!\n";
	exit;
    }
}

sub check_file {
    my ($file) = @_;
    unless(-e $file) {
	print "$file does not exit!\n";
	return 0;
    }
    return 1;
}

sub chk_dir {
    my ($filedir) = @_;

    unless($filedir =~ /\/$/) {
	print "$filedir does not end with \"\/\"\! Exit!\n";
	exit;
    }
    unless(-e $filedir) {
	print "$filedir does not exit, please check!Exit!\n";
	exit;	
    }	
}

sub get_bcrd {
    my ($datfile, $scandir) = @_;
    
    my $micronum;
    if($datfile =~ m/\/(\d+)\.dat/) {
	$micronum = $1;
    } 
    
    my $fullname = $scandir . $micronum;
    my @bcrd = <$fullname*.bcrd>;
    return $bcrd[0];
}

sub get_value {
    my @arr = split /,/, $_[0];
    my $ret = $arr[$_[1]];
    return $ret;
}	

sub by_index {
    my $id1 = get_value($a, 0);
    my $id2 = get_value($b, 0);	
    $id1 <=> $id2;
}

sub create_dir {
    if (opendir (DIR, $_[0])) {
	closedir (DIR);
    } else {
	mkdir ($_[0]) or die 
	    "Cannot create $_[0] directory, please create\
 it manually, auto3dem quit!\n";
    }
}
