      subroutine get_ptle_ioom(nparam, nimg_per_param, filename_len, 
     $     param_fnames, oddeven, nptles, nbox, indx, 
     $     theta, phi, omega, x, y, mag)
      
C     Get indices origins, orientations, and magnifications (ioom) for
C     multiple particle parameter files
C
C     nparam         Number of parameter files
C     nimg_per_param Number of image files per parameter file
C     filename_len   Maximum number of characters in file name
C     param_fnames   Array of file names
C     oddeven        Flag indicating whether 'odd', 'even', or all images
C                    should be counted. Note that designation of even or
C                    odd is continuous across all files
C     nptles         Total number of particle images
C     nbox           Box size for particle images
C     indx           Array of particle indices used to number images
C                    in the boxed image files
C     theta          Theta orientation angles
C     phi            Phi orientation angles
C     omega          Omega orientation angles
C     x              x-coordinates of ptle origins
C     y              y-coordinates of ptle origins
C     mag            Magnifications

      integer, intent(in) :: nparam, filename_len, oddeven, nptles
      integer, intent(in) :: nbox(nparam)
      integer, intent(in) :: nimg_per_param(nparam)
      character(len=filename_len), intent(in) :: param_fnames(nparam)

      integer, intent(out) :: indx(nptles)
      real, intent(out)    :: theta(nptles), phi(nptles), omega(nptles)
      real, intent(out)    :: x(nptles), y(nptles), mag(nptles)

      integer i, j, indx_temp, ios, count
      integer, save :: skipone = 1
      real theta_temp, phi_temp, omega_temp, x_temp, y_temp, mag_temp
      character(len=1) :: heading

      count = 0
      do i=1,nparam
         open(unit=17, file=param_fnames(i))

         do j=1,nimg_per_param(i)
            read(17,'(A)') heading
            read(17,'(A)') heading
            
            do
               read(17,*,iostat=ios) indx_temp, theta_temp, phi_temp,
     $              omega_temp, x_temp, y_temp, mag_temp
               
               if(ios .ne. 0) exit ! Break out of loop
               
               if(  (oddeven .ne. 1 .and. oddeven .ne. 2) .or.
     $              (oddeven == 1 .and. skipone == 1) .or.
     $              (oddeven == 2 .and. skipone == 0) ) then
                  count = count + 1
                  
                  if(x_temp <= 1)      x_temp = nbox(i)/2
                  if(y_temp <= 1)      y_temp = nbox(i)/2
                  if(mag_temp <= 0.01) mag_temp = 1.0
                  
                  indx(count)  = indx_temp
                  theta(count) = theta_temp
                  phi(count)   = phi_temp
                  omega(count) = omega_temp
                  x(count)     = x_temp
                  y(count)     = y_temp
                  mag(count)   = mag_temp
               endif
               
               skipone = mod(skipone+1, 2) 
            enddo
            backspace(17) ! step back one line to account for failed read
         enddo
         
         close(17)
      enddo
      
      return
      end subroutine get_ptle_ioom
