/* used for the interface for C and Fortran call */
/*   - Yongchang Ji */
/* add by ji for cftfit*/
#include <stdio.h>
#include <math.h>
#include "fitutil.h"
#include "../include/common.h"

/* 
   ca2d_AVG: an array of avg value at certain fft radius
   ca2d_STD: an array of std value at certain fft radius
   poly_paras : initial parameters for Background, Decay, defocus, angle also used for return values
   poly_order : is the size of poly_paras
   poly_orderb: is the size of parameters for Background
   poly_orderd: is the size of parameters for Decay
   wavelength :
   res_hi     :
   hkl_max    : is the size of have fft size
   A          : Percentage of Amplitude Contrast
   Cs         : Spherical aberration constant
   pixel_size : 
   chisq      : return value
*/
/* define used for fortran call method */

void cmpt_fit(double ca2d_avg[], double ca2d_std[], 
	      int * hkl_max, double * res_hi,
	      double poly_paras[], int * poly_order, 
	      int * poly_orderb, int * poly_orderd,
	      double * wavelength, double * A, double * Cs, 
	      double * pixel_size, double * chisq, 
	      float *auto2DdrawBuf, float *auto2DdataBuf)
{
     cmpt_fit_(ca2d_avg, ca2d_std, hkl_max, res_hi, 
               poly_paras, poly_order, poly_orderb, poly_orderd,
               wavelength, A, Cs, pixel_size, chisq, 
	       auto2DdrawBuf, auto2DdataBuf);
}

/* real code part for ctf fiting */
/* note ca2d_avg ca2d_std are stored in Fortran format
   ca2d_avg(0:hkl_max, -hkl_max + hkl_max + 1)
*/
void cmpt_fit_(double ca2d_avg[], double ca2d_std[], int * hkl_max, 
	       double * res_hi,
	       double poly_paras[], int * poly_order, int * poly_orderb, 
	       int * poly_orderd,
	       double * wavelength, double * A, double * Cs, 
	       double * pixel_size, double * chisq, 
	       float *auto2DdrawBuf, float *auto2DdataBuf)
{
  
  int    *ca2d_id;
  double **covar, **alpha;
  double big=100000000.0, small=0.000000001, PI=3.14159265358979323;
  double chisqold1, fw, tmp3, tmp1, tmp2;
  double chisqold2, angleold, def1old,def2old;
  double alamda, defmajor, defminor;
  double angle, theta1, deg_rad, rad_deg, chisqtmp, angtmp;
  int    *ia, i, j, k, ma, masq, lenx, leny, exchange;
  double annlow, annhigh, ratio, annlow_max, annlow_min;
  double annhigh_max, annhigh_min, radius, def, ann;
  int kk;  

  deg_rad = PI / 180.0;
  rad_deg = 180.0 / PI;
  
  i = *poly_order;
  defmajor = poly_paras[i-3];
  defminor = poly_paras[i-2];
  angle   = poly_paras[i-1] * deg_rad; /* convert to radian */
  poly_paras[i-1]  = angle; /* save back angle in radian instead of degree*/
  
  /* allocate memory for store whether wanted data or not, ca2d_id[y][x] */
  lenx = * hkl_max + 1;
  leny = 2* (* hkl_max) + 1;
  ca2d_id = (int *) malloc ( lenx * leny * sizeof(int ));
  
  /* compute the annulus of first peak and annuals high*/
  fw = sqrt((defmajor - sqrt(defmajor*defmajor - 
			     (*Cs)*(*wavelength)*2.0))/(*Cs));
  annlow_max = fw*((*pixel_size)*2*(*hkl_max))/(*wavelength);
  if (annlow_max < 1.0) annlow_max = 1.0;
  fw = sqrt((defminor - sqrt(defminor*defminor - 
			     (*Cs)*(*wavelength)*2.0))/(*Cs));
  annlow_min = fw*((*pixel_size)*2*(*hkl_max))/(*wavelength);
  if (annlow_min < 1.0) annlow_min = 1.0;
  
  if (annlow_min > annlow_max) {
    annhigh_min = 2*(*hkl_max)*(*pixel_size)/(*res_hi);
    if( annhigh_min > (*hkl_max)) annhigh_min = (*hkl_max);
    annhigh_max = annhigh_min * annlow_max / annlow_min;
    annlow = annlow_max;
    annhigh = annhigh_min;
  } else {
    annhigh_max = 2*(*hkl_max)*(*pixel_size)/(*res_hi);
    if( annhigh_max > (*hkl_max)) annhigh_max = (*hkl_max);
    annhigh_min = annhigh_max * annlow_min / annlow_max;
    annlow = annlow_min;
    annhigh = annhigh_max;
  }
  ratio = annhigh_max / annlow_max;
  
  /* printf("wavelength = %f, annlow_min = %f, annlow_max= %f, 
     annhigh_min= %f, annhigh_max= %f\n",
     *wavelength, annlow_min, annlow_max, annhigh_min, annhigh_max);*/
  
  /* find which points is located in the elipse of annlow and annhigh */
  for (j=0; j<leny; j++) {
    k = j-lenx+1;   /* Y axis from -hkl_max to hkl_max */
    for (i=0; i<lenx; i++) {
      ca2d_id[j*lenx + i] = 0;
      radius = sqrt(i*i + k*k);
      if ( radius >= annlow && radius <= annhigh) {
	if (i>0) theta1 = atan2(k,i);
	else {
	  if (k >=0) theta1 = PI/2;
	  else theta1 = -PI/2;
	}
	theta1 -= angle;
	tmp1    = cos(theta1);
	tmp2   = sin(theta1);
	def    = defmajor * tmp1 * tmp1 + defminor * tmp2 * tmp2;
	fw     = sqrt((def - sqrt(def*def - (*Cs)*(*wavelength)*2.0))/(*Cs));
	ann    = fw*((*pixel_size)*2*(*hkl_max))/(*wavelength);
	if (radius>=ann && radius<=ann*ratio)  ca2d_id[j*lenx + i] = 1;
      }
    }
  }
  
  ma  = *poly_order;
  ia  = (int *) malloc (ma * sizeof(int ));
  covar = (double **) malloc (ma * sizeof(double ));
  alpha = (double **) malloc (ma * sizeof(double ));
  masq = ma * ma;
  covar[0] = (double *) malloc (masq * sizeof(double ));
  for (i=1; i<ma; i++) covar[i] = covar[i-1] + ma;
  alpha[0] = (double *) malloc (masq * sizeof(double ));
  for (i=1; i<ma; i++) alpha[i] = alpha[i-1] + ma;
  
  /***********************************************************************
   * In this iteration session, all parameters except angle are adjusted *
   ***********************************************************************/
  for(i=0; i<ma-1; i++) ia[i] = 1;
  ia[ma-1] = 0;
  for (kk=0;kk<1; kk++){
    alamda = -1.0;
    for (;;) {
      yji_mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
		  covar, alpha, chisq, yji_fpoly_ctf, &alamda,
		  *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
		  *A, *Cs, *pixel_size, auto2DdrawBuf, auto2DdataBuf);
      if(alamda>big || alamda<small) break;
    }
    alamda = 0.0;
    yji_mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
                covar, alpha, chisq, yji_fpoly_ctf, &alamda,
                *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
                *A, *Cs, *pixel_size, auto2DdrawBuf, auto2DdataBuf);
    
    /*  for (i=0; i<*poly_orderb; i++)
	printf("\na[%d]= %e",i, poly_paras[i]);
	for (i=*poly_orderb; i<*poly_orderb+*poly_orderd; i++)
	printf("\nb[%d]= %e",i, poly_paras[i]);*/
    printf ("\n\ndefocus major: %e \n",  poly_paras[*poly_order-3]);
    printf ("defocus minor: %e \n",  poly_paras[*poly_order-2]);
    printf ("angle: %e \n",  poly_paras[*poly_order-1]*rad_deg);
    printf ("chisq: %e \n\n",  *chisq);
  }
  /************************************************************             
   * In this iteration session, the best angle are determined *
   ************************************************************/
  for(i=0; i<ma-1; i++) ia[i] = 0;
  ia[ma-1]=1;
  /* Wei research all possibilities of angles */
  angle = poly_paras [*poly_order-1];
  angtmp = angle;
  chisqtmp = *chisq;
  for (i=0; i<90; i++) {
    poly_paras [*poly_order-1] = angle + deg_rad*(i-45.0);
    alamda = -1.0;
    for (;;) {
      yji_mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
		  covar, alpha, chisq, yji_fpoly_ctf, &alamda,
		  *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
		  *A, *Cs, *pixel_size, auto2DdrawBuf, auto2DdataBuf);
      if(alamda>big || alamda<small) break;
    }
    alamda = 0.0;
    yji_mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
                covar, alpha, chisq, yji_fpoly_ctf, &alamda,
                *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
                *A, *Cs, *pixel_size, auto2DdrawBuf, auto2DdataBuf);
    if (*chisq <chisqtmp) {
      chisqtmp = *chisq;
      angtmp = poly_paras[*poly_order-1];
    }
    i++;i++;i++;i++;/* calculate this cycle every 5 degrees to save time */
  }
  poly_paras[*poly_order-1] = angtmp;
  printf ("\nThe new angle is founded!\n");
  printf ("		angle: %e \n",  poly_paras[*poly_order-1]*rad_deg);
  /**********************************************************************
   now every thing except angle is recalculated
  ***********************************************************************/
  for(i=0; i<ma-1; i++) ia[i] = 1;
  ia[ma-1] = 0;
  alamda = -1.0;
  for (;;) {
    yji_mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, 
		ia, *poly_order,
		covar, alpha, chisq, yji_fpoly_ctf, &alamda,
		*poly_orderb, *poly_orderd, *wavelength, 
		*hkl_max, *res_hi,
		*A, *Cs, *pixel_size, 
		auto2DdrawBuf, auto2DdataBuf);
    
    printf("Third pass [%f]\n",alamda);
    
    if(alamda>big || alamda<small) break;
  }
  alamda = 0.0;
  yji_mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
	      covar, alpha, chisq, yji_fpoly_ctf, &alamda,
	      *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
	      *A, *Cs, *pixel_size, auto2DdrawBuf, auto2DdataBuf);
  
  for (i=0; i<*poly_orderb; i++)
    printf("\na[%d]= %e",i, poly_paras[i]);
  for (i=*poly_orderb; i<*poly_orderb+*poly_orderd; i++)
    printf("\nb[%d]= %e",i, poly_paras[i]);
  printf ("\n\ndefocus major: %e \n",  poly_paras[*poly_order-3]);
  printf ("defocus minor: %e \n",  poly_paras[*poly_order-2]);
  printf ("angle: %e \n",  poly_paras[*poly_order-1]*rad_deg);
  printf ("chisq: %e \n\n",  *chisq);
  /*end here refine angle*/
  
  free(ca2d_id);
  free(covar[0]);
  free(covar);
  free(alpha[0]);
  free(alpha);
  free(ia);
}


#define SWAP1(a,b) {swap=(a);(a)=(b);(b)=swap;}

void yji_covrst(double **covar, int ma, int ia[], int mfit)
     /* Expand in storage the covariance matrix covar, so as to
	take into account parameters that are being held fixed. 
	program: Numerical Recipe in C, p39
	modified by Wei Zhang, change float into double*/
     
{
  int i, j, k;
  double swap;
  
  for (i=mfit;i<ma;i++)
    for (j=0;j<=i;j++) covar[i][j]=covar[j][i]=0.0;
  k=mfit;
  for (j=ma-1;j>=0;j--) {
    if (ia[j]) {
      k--;
      for (i=0;i<ma;i++) SWAP1(covar[i][k], covar[i][j])
      for (i=0;i<ma;i++) SWAP1(covar[k][i], covar[j][i])
						}
  }
}



#define DEBUG1 0

void yji_fpoly_ctf(int idx, int idy, double a[], 
		   double *ymode, double dyda[], 
		   int na, 
		   int order_poly1,
		   int order_poly2, 
		   double  wavelength, 
		   int idim_half, 
		   double  A, 
		   double  Cs,
		   double  pixel_size )
{
  int i, j, k, orderb, orderd;
  double  arg1, arg2, dvalue, defmajor, defminor, angle, theta1;
  double  fw, fw2, theta, def, PI, ctf, ctf2, tmp1, tmp2;
  double  cosph, sinph, costh, sinth, dctfdth;
  
  PI=3.141592653589793238;
  angle    = a[na-1];
  defminor = a[na-2];
  defmajor = a[na-3];
  
  if (idx!=0) theta1 = atan2(idy,idx);
  else {
    if (idy >=0) theta1 = PI/2;
    else theta1 = -PI/2;
  }
  /*        printf("	   theta1 = %f\n",theta1);*/
  theta1 -= angle;
  cosph = cos(theta1);
  sinph = sin(theta1);
  /*  printf("	 angle = %f\n",angle*180/3.14159265358979);
      printf("	 theta1 = %f\n",theta1);
      printf("	 cosph = %f\n",cosph);
      printf("	 sinph = %f\n",sinph);*/
  def     = defmajor*cosph*cosph + defminor*sinph*sinph;
  
  /*printf("	 def = %f\n",def);*/
  fw =wavelength*sqrt(idx*idx + idy*idy)/(idim_half*2.0* pixel_size );
  fw2= fw*fw;
  theta = PI*2.0*(Cs*fw2*fw2/4.0-def*fw2/2.0)/wavelength;
  costh = cos(theta);
  sinth = sin(theta);
  ctf = sqrt(1.0-A*A)*sinth-A*costh;
  ctf2 = ctf*ctf;
  dctfdth = sqrt(1.0-A*A)*costh+A*sinth;
  
  orderb = floor(sqrt(2*order_poly1)) -1;
  orderd = floor(sqrt(2*order_poly2)) -1;
  
  /* compute B + ctf*ctf*D
     we formulate the B and D in the order of x, 2nd order is y, 
     such as order 2:  a0 + a1*y + a2*y*y + a3*x + a4*x*y + a5*x*x  
     such as order 3:  a0 + a1*y + a2*y*y + a3*y*y*y + a4*x + a5*x*y + 
                            a6*x*y*y + a7*x*x +a8*x*x*y + a9*x*x*x 
  */
  *ymode=0.0;
  arg1 = 1;
  k    = 0;
  for (i = 0; i<= orderb; i++) {
    arg2  = 1;
    for (j = 0; j <= orderb - i; j++){
      dyda[k] = arg1 * arg2;
      *ymode += a[k] * dyda[k];
      arg2   *= idy;
      k++;
    }
    arg1  *= idx;
  }
  
  dvalue = 0.0;
  arg1 = 1;
  for (i = 0; i<= orderd; i++) {
    arg2  = 1;
    for (j = 0; j <= orderd - i; j++){
      dyda[k] = ctf2 * arg1 * arg2;
      *ymode += a[k] * dyda[k];
      dvalue += a[k] * arg1 * arg2;
      arg2   *= idy;
      k++;
    }
    arg1  *= idx;
  }
  
  /* compute using defocus major */
  dyda[na-3]= -2.0*PI*ctf*dvalue*fw2*dctfdth*cosph*cosph /wavelength;
  /* compute using defocus minor */
  dyda[na-2]= -2.0*PI*ctf*dvalue*fw2*dctfdth*sinph*sinph /wavelength ;
  /* compute using angle */
  dyda[na-1]= -4.0*PI*ctf*dvalue*fw2*(defmajor-defminor)*
    dctfdth*sinph*cosph /wavelength;
  if(DEBUG1) {   
    printf("\nX is %d, \n Y is %d\n", idx, idy);
    printf("   ymode = %f\n", *ymode);
    for(i=0;i<na; i++)  
      printf("   a[%d] = %f\n",i, a[i]);
    for(i=0;i<na; i++)
      printf("   dyda[%d] = %f\n",i, dyda[i]);
  } 
}
/* The routine for Gauss-Jordan elimination with full pivoting */

#define SWAP(a,b) {temp=(a); (a)=(b); (b)=temp;}

void yji_gaussj(double **a, int n, double **b, int m)
     /* Linear equation solution by Gauss-Jordan elimination,
	a[1..n][1..n] is the input matrix. b[1..n][1..m] is input
	containing the m right-hand side vectors. On output, a is
	replaced by its matrix inverse, and b is replaced by the
	corresponding set of solution vectors.  program: Numerical
	Recipe in C, p39 modified by Wei Zhang, change float into
	double*/
{
  int *indxc, *indxr, *ipiv;
  int i, icol, irow, j, k, l, ll;
  double big, dum, pivinv, temp;
  
  indxc=(int *) malloc (n * sizeof(int));
  indxr=(int *) malloc (n * sizeof(int));
  ipiv =(int *) malloc (n * sizeof(int));
  for (j=0;j<n;j++) ipiv[j]=0;
  for (i=0;i<n;i++) {
    big=0.0;
    for (j=0;j<n;j++)
      if (ipiv[j] != 1) 
        for (k=0;k<n;k++) {
          if (ipiv[k]==0) {
            if (fabs(a[j][k]) >=big) {
              big=fabs(a[j][k]);
              irow=j;
              icol=k;
            }
          } else if (ipiv[k]>1)
	    {
	      fprintf(stderr, "yji_gaussj: Singular Matrix-1\n");
	      exit(1);
	    };
        }
    ++(ipiv[icol]);
  
    if (irow != icol) {
      for (l=0;l<n;l++) SWAP(a[irow][l], a[icol][l])
      for (l=0;l<m;l++) SWAP(b[irow][l], b[icol][l])
    }
    indxr[i]=irow;
    indxc[i]=icol;
    
    if (a[icol][icol] == 0.0) {
      fprintf(stderr, "yji_gaussj: Singular Matrix-2\n");
      exit(1);
    };
    
    pivinv=1.0/a[icol][icol];
    a[icol][icol]=1.0;
    for (l=0;l<n;l++) a[icol][l] *= pivinv;
    for (l=0;l<m;l++) b[icol][l] *= pivinv;
    for (ll=0;ll<n;ll++)
      if (ll != icol) {
        dum=a[ll][icol];
        a[ll][icol]=0.0;
        for (l=0;l<n;l++) a[ll][l] -= a[icol][l]*dum;
        for (l=0;l<m;l++) b[ll][l] -= b[icol][l]*dum;
      }
  }
  
  for (l=n-1;l>=0;l--) {
    if (indxr[l] != indxc[l])
      for (k=0;k<n;k++)
        SWAP(a[k][indxr[l]], a[k][indxc[l]]);
  }
  
  free(ipiv);
  free(indxr);
  free(indxc);
}

void yji_mrqcof(int id[], 
		double y[], 
		double sig[], 
		double a[], 
		int ia[],
		int ma, 
		double **alpha, 
		double beta[], 
		double *chisq,
		void (*funcs)(int, int, double [], double *, double [], 
			      int, int, int, double, int, 
			      double, double, double),
		int num_gauss_bkg, 
		int num_gauss_ctf, 
		double wavelength, 
		int idim_half, 
		double res_hi, 
		double A,
		double Cs, 
		double pixel_size,
		float *auto2DdrawBuf,
		float *auto2DdataBuf)
     
     /* Used by yji_mrqmin to evaluate the linearized fitting matrix
	alpha, and vector beta and calculate chi square
	
	program: Numerical Recipe in C, p686 modified by Wei Zhang,
	pass more parameters and change float into double*/ 
{
  int i, j, k, l, m, mfit, lenx, leny, len, idx, idy;
  double ymod, wt, sig2i, dy, *dyda;
  double theta1, PI, cosph, sinph, fw, ann, defmajor, defminor;
  double radius, angle, def;
  double tmp1, tmp2, annminor;
  
  /* wei add several parameters for re-calculating sig from averaged FFTs*/
  double *cdavg, *cdstd, *cdcnt, pnt, tmp;
  int    irad;
  dyda=(double *) malloc (ma * sizeof(double));
  mfit = 0;
  for (j=0;j<ma;j++)
    if(ia[j]) mfit++;
  for(j=0;j<mfit;j++) {
    for (k=0;k<=j;k++) alpha[j][k]=0.0;
    beta[j]=0.0;
  }
  *chisq=0.0;
  PI=3.141592653589793238;
  lenx = idim_half + 1;
  leny = 2*idim_half + 1;
  len = lenx * leny;
  /* allocate memory for  recalculate sig */
  
  /*  caculate the max annuals at the fisrt elipse */
  /*  recalculate sig */
  
  for (i=0;i<len; i++) { 
    if (id[i]==1) {
      idx = i%lenx;
      idy = i/lenx - idim_half;
      (*funcs)(idx, idy, a, &ymod, dyda, ma, 
	       num_gauss_bkg, num_gauss_ctf, wavelength, idim_half,
               A, Cs, pixel_size);    
      if(idx < 256 && idy < 256 && idy > -256)
	{/* This is it */
	  auto2DdataBuf[(((idy+256)*256)+idx)] = (float)y[i];
	  auto2DdrawBuf[(((idy+256)*256)+idx)] = (float)ymod;
	}/* This is it */
      else
	printf("auto2DdrawBuf not found [%d][%d]\n",idx,idy);
      if(sig[i]==0) printf("sig is 0!\n");
      sig2i=1.0/(sig[i]*sig[i]);    
      dy=y[i]-ymod;
      for (j=-1,l=0;l<ma;l++) {
	if (ia[l]) {
	  wt=dyda[l]*sig2i;
	  for (j++,k=0,m=0;m<=l;m++)
	    if(ia[m]) alpha[j][k++]+=wt*dyda[m];
	  beta[j] += dy*wt;
	}
      }
      
      radius = sqrt(idx*idx + idy *idy);
      *chisq +=dy*dy*sig2i / (radius*radius);
    }
  }
  for (j=1;j<mfit;j++)
    for (k=0;k<j; k++) alpha[k][j]=alpha[j][k];
}

void yji_mrqmin(int x[], double y[], double sig[], double a[], int ia[],
		int ma, double **covar, double **alpha, double *chisq,
		void (*funcs)(int, int, double [], double *, 
			      double [], int, int, int, double,
			      int, double, double, double), 
		double *alamda, int num_gauss_bkg, int num_gauss_ctf, 
		double wavelength,
		int idim_half, double res_hi, double A, double Cs, 
		double pixel_size, 
		float *auto2DdrawBuf, float *auto2DdataBuf)
     /*program: Numerical Recipe in C, p687 modified by Wei Zhang,
       pass more paramerters and change float into double*/
{
  int j, k, l;
  static int mfit;
  static double ochisq, *atry, *beta, *da, **oneda;
  
  if (*alamda < 0.0) {
    atry=(double *) malloc (ma * sizeof(double ));
    beta=(double *) malloc (ma * sizeof(double ));
    da  =(double *) malloc (ma * sizeof(double ));
    for (mfit=0,j=0;j<ma;j++)
      if (ia[j]) mfit++;
    oneda=(double **) malloc ( mfit * sizeof(double ));
    oneda[0] = (double *) malloc (mfit* sizeof(double ));
    for (j=1; j<mfit; j++) oneda[j] = oneda[j-1] + 1 ;
    
    *alamda=0.001;      
    yji_mrqcof(x,y,sig,a,ia,ma,alpha,beta,chisq,
	       funcs,num_gauss_bkg,num_gauss_ctf,
	       wavelength, idim_half, res_hi, A, Cs, pixel_size,
	       auto2DdrawBuf, auto2DdataBuf);
    ochisq=(*chisq);
    for(j=0;j<ma;j++) atry[j]=a[j];
  }
  
  for (j=0;j<mfit;j++) {
    for (k=0;k<mfit;k++) covar[j][k]=alpha[j][k];
    covar[j][j]=alpha[j][j]*(1.0+(*alamda));
    oneda[j][0]=beta[j];
  }
  yji_gaussj(covar, mfit, oneda, 1);
  
  for (j=0;j<mfit;j++) da[j]=oneda[j][0];
  if (*alamda == 0.0) {
    yji_covrst(covar, ma, ia, mfit);
    free(oneda[0]);
    free(oneda);
    free(da);
    free(atry);
    return;
  }
  
  for (j=0, l=0; l<ma; l++)
    if(ia[l]) atry[l]=a[l]+da[j++];
  yji_mrqcof(x,y,sig,atry,ia,ma,covar,da,chisq,
	     funcs,num_gauss_bkg,num_gauss_ctf,
	     wavelength, idim_half, res_hi, A, Cs, pixel_size,
	     auto2DdrawBuf, auto2DdataBuf);
  
  if(*chisq<ochisq) {
    *alamda *= 0.1;
    ochisq=(*chisq);
    for (j=0;j<mfit; j++) {
      for (k=0;k<mfit;k++) alpha[j][k]=covar[j][k];
      beta[j]=da[j];
    }
    for(l=0;l<ma;l++) a[l]=atry[l];
  } else {
    *alamda *= 10.0;
    *chisq=ochisq;
  }
}



void stuffrobembuf_ (float  *auto2DdrawBuf, 
		     double *a, 
		     int    *naCall,
		     double *waveCall, double *pixSizCall,
		     double *CsCall, double *ACall,
		     int *poly1Call, int *poly2Call)
{
  // function name: stuffrobembuf_
  // Do whatever it takes to make a display buffer for robem that
  // describes: s = b + ctf^2 * d
  
  int     i, j, k, orderb, orderd, idx, idy, idim_half;
  double  arg1, arg2, dvalue, defmajor, defminor, angle, theta1;
  double  fw, fw2, theta, def, PI, ctf, ctf2, tmp1, tmp2;
  double  cosph, sinph, costh, sinth, dctfdth;
  double  ymode, *dyda;
  double  wavelength, pixel_size, Cs, A;
  int     na, order_poly1, order_poly2, x, y;
  float   minF,maxF,scaleF;
  int     pixelX, pixelY;
  
  na          = *naCall;
  wavelength  = *waveCall;
  pixel_size  = *pixSizCall;
  Cs          = *CsCall;
  A           = *ACall;
  order_poly1 = *poly1Call;
  order_poly2 = *poly2Call;    
  
  PI=3.141592653589793238;
  angle    = a[na-1];
  defminor = a[na-2];
  defmajor = a[na-3];
  
  dyda=(double *) malloc (na * sizeof(double));
  for(i=0;i<na;i++)
    dyda[i] = 0.0;
  
  for(y = 0 ; y < 512 ;y ++) {
    
    idy = 256 - y;
    if(idy < 1)
      idy = 0 - idy;
    
    for(x = 0 ; x < 256 ;x ++) {
      
      idx = x + 256;
      
      if (idx!=0) theta1 = atan2(idy,idx);
      else {
	if (idy >=0) theta1 = PI/2;
	else theta1 = -PI/2;
      }
      /*        printf("	   theta1 = %f\n",theta1);*/
      theta1 -= angle;
      cosph = cos(theta1);
      sinph = sin(theta1);
      /*  printf("	 angle = %f\n",angle*180/3.14159265358979);
	  printf("	 theta1 = %f\n",theta1);
	  printf("	 cosph = %f\n",cosph);
	  printf("	 sinph = %f\n",sinph);*/
      def = defmajor*cosph*cosph + defminor*sinph*sinph;
      
      /*printf("	 def = %f\n",def);*/
      
      idim_half = 256; /* I think this is right for our 512 display */
      fw =wavelength*sqrt(idx*idx + idy*idy)/(idim_half*2.0* pixel_size );
      fw2= fw*fw;
      theta = PI*2.0*(Cs*fw2*fw2/4.0-def*fw2/2.0)/wavelength;
      costh = cos(theta);
      sinth = sin(theta);
      ctf = sqrt(1.0-A*A)*sinth-A*costh;
      ctf2 = ctf*ctf;
      dctfdth = sqrt(1.0-A*A)*costh+A*sinth;
      
      orderb = floor(sqrt(2*order_poly1)) -1;
      orderd = floor(sqrt(2*order_poly2)) -1;
      
      /* compute B + ctf*ctf*D
	 we formulate the B and D in the order of x, 2nd order is y, 
	 such as order 2:  a0 + a1*y + a2*y*y + a3*x + a4*x*y + a5*x*x  
	 such as order 3:  a0 + a1*y + a2*y*y + a3*y*y*y + a4*x + a5*x*y + 
	                   a6*x*y*y + a7*x*x +a8*x*x*y + a9*x*x*x 
      */

      ymode=0.0;
      arg1 = 1;
      k    = 0;
      for (i = 0; i<= orderb; i++) {
	arg2  = 1;
	for (j = 0; j <= orderb - i; j++){
	  dyda[k] = arg1 * arg2;
	  ymode += a[k] * dyda[k];
	  arg2   *= idy;
	  k++;
	}
	arg1  *= idx;
      }
      
      dvalue = 0.0;
      arg1 = 1;
      for (i = 0; i<= orderd; i++) {
	arg2  = 1;
	for (j = 0; j <= orderd - i; j++) {
	  dyda[k] = ctf2 * arg1 * arg2;
	  ymode += a[k] * dyda[k];
	  dvalue += a[k] * arg1 * arg2;
	  arg2   *= idy;
	  k++;
	}
	arg1  *= idx;
      }
      
      auto2DdrawBuf[((y*256)+x)] = (float) ymode;
      
    }
  }
  
  minF = maxF = auto2DdrawBuf[0];
  for(i=0;i<(256*512);i++) {
    minF = MIN(minF, auto2DdrawBuf[i]);
    maxF = MAX(maxF, auto2DdrawBuf[i]);
  }
  scaleF = 200.0 / (maxF - minF);
  
  for(y = 0 ; y < 512 ;y ++) {
    for(x = 0 ; x < 256 ;x ++) {
      auto2DdrawBuf[((y*256)+x)] = 10.0 +
	(float)((auto2DdrawBuf[((y*256)+x)] - minF) * scaleF);
    }
  }
  
} // end function stuffrobembuf_
